#include "shmem.h"

#include <sys/ipc.h>
#include <sys/shm.h>
#include <sched.h>

#include <signal.h>

#include <sstream>

#include <map>

int spawn(const char *l_Command)
{
	int l_PID = fork();
	if(l_PID == -1) return 0;
	if(l_PID == 0) {
		char *argv[4];
		argv[0] = "sh";
		argv[1] = "-c";
		argv[2] = (char *)l_Command;
		argv[3] = 0;
		execve("/bin/sh", argv, environ);
		exit(127);
	}
	return l_PID;
}

int main(int argc, char *argv[])
{
	unsigned numprocs = atoi(argv[1]);
	int shmid  = shmget(SHMKEY, numprocs * sizeof(struct ShmBlock), IPC_CREAT | IPC_EXCL | 0666);
	void *shmem_ptr = shmat(shmid, 0, 0);
	std::map<unsigned, int> l_PIDMap;
	for(unsigned idx = 0; idx < numprocs; ++idx) {
		struct ShmBlock *shmblock = &((struct ShmBlock*)shmem_ptr)[idx];
		shmblock->state = UNDEFINED;
		std::ostringstream l_Command("");
		l_Command << getenv("PWD") << "/process " << idx << " " << numprocs;
		l_PIDMap[idx] = spawn(l_Command.str().c_str());
		while(shmblock->state != INIT_REQ) sched_yield();
		shmblock->state = INIT_ACK;
	}
	for(unsigned idx = 0; idx < numprocs; ++idx) kill(l_PIDMap[idx], SIGTERM);
	shmdt(shmem_ptr);
	struct shmid_ds l_ShmID_DS;
	shmctl(shmid, IPC_RMID, &l_ShmID_DS);
	return 0;
}

